Package: ExpHunterSuite
Title: Package For The Comprehensive Analysis Of Transcriptomic Data
Version: 1.17.0
Authors@R: 
    c(person(given = "James",
           family = "Perkins",
           role = c("aut", "cre"),
           email = "jimrperkins@gmail.com",
           comment = c(ORCID = "0000-0003-4108-096X")),
    person(given = "Pedro",
           family = "Seoane Zonjic",
           role = c("aut"),
           email = "seoanezonjic@uma.es",
           comment = c(ORCID = "0000-0002-3020-1415")),
    person(given = "Fernando",
           family = "Moreno Jabato",
           role = c("aut"),
           email = "fmjabato@gmail.com",
           comment = c(ORCID = "0000-0001-7498-1962")),     
    person(given = "José",
           family = "Córdoba Caballero",
           role = c("aut"),
           email = "josecordoba1995@gmail.com",
           comment = c(ORCID = "0000-0002-1821-5742")),      
    person(given = "Elena",
           family = "Rojano Rivera",
           role = c("aut"),
           email = "elenarojano@outlook.com",
           comment = c(ORCID = "0000-0002-2678-710X")),
    person(given = "Rocio",
           family = "Bautista Moreno",
           role = c("aut"),
           email = "rociobm@uma.es",
           comment = c(ORCID = "0000-0003-1685-8119")),
    person(given = "M. Gonzalo",
           family = "Claros",
           role = c("aut"),
           email = "claros@uma.es",
           comment = c(ORCID = "0000-0002-0112-3550")),          
    person(given = "Isabel",
           family = "Gonzalez Gayte",
           role = c("aut"),
           email = "isabel.ggayte@gmail.com"),         
    person(given = "Juan Antonio",
           family = "García Ranea",
           role = c("aut"),
           email = "ranea@uma.es",
           comment = c(ORCID = "0000-0003-0327-1837"))
    )
Description: The ExpHunterSuite implements a comprehensive protocol for the analysis of transcriptional data using established *R* packages and combining their results. It covers all key steps in DEG detection, CEG detection and functional analysis for RNA-seq data. It has been implemented as an R package containing functions that can be run interactively. In addition, it also contains scripts that wrap the functions and can be run directly from the command line.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
biocViews: GeneExpressionWorkflow
Suggests: optparse, PerformanceAnalytics, naivebayes, reshape2,
        org.Hs.eg.db, org.Mm.eg.db, testthat (>= 3.0.0)
VignetteBuilder: knitr
Imports: ReactomePA, limma, edgeR, NOISeq, biomaRt, topGO, diffcoexp,
        DT, ggplot2, stringr, WGCNA, dplyr, AnnotationDbi,
        BiocGenerics, enrichplot, rmarkdown, stats, Biobase, DESeq2,
        ROCR, data.table, knitr, magrittr, SummarizedExperiment,
        miRBaseVersions.db, grDevices, graphics, utils, BiocParallel,
        MKinfer, matrixStats, ggupset, rlang, plyr, tidyr, GO.db,
        Matrix, fastcluster, DOSE, heatmaply, EnhancedVolcano, ggrepel,
        clusterProfiler, GenomicRanges, GenomicFeatures, tximport,
        annotatr, ggridges, FactoInvestigate, FactoMineR
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/ExpHunterSuite
git_branch: devel
git_last_commit: d31b29e
git_last_commit_date: 2025-04-15
Repository: Bioconductor 3.22
Date/Publication: 2025-06-27
NeedsCompilation: no
Packaged: 2025-06-27 17:07:50 UTC; biocbuild
Author: James Perkins [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4108-096X>),
  Pedro Seoane Zonjic [aut] (ORCID:
    <https://orcid.org/0000-0002-3020-1415>),
  Fernando Moreno Jabato [aut] (ORCID:
    <https://orcid.org/0000-0001-7498-1962>),
  José Córdoba Caballero [aut] (ORCID:
    <https://orcid.org/0000-0002-1821-5742>),
  Elena Rojano Rivera [aut] (ORCID:
    <https://orcid.org/0000-0002-2678-710X>),
  Rocio Bautista Moreno [aut] (ORCID:
    <https://orcid.org/0000-0003-1685-8119>),
  M. Gonzalo Claros [aut] (ORCID:
    <https://orcid.org/0000-0002-0112-3550>),
  Isabel Gonzalez Gayte [aut],
  Juan Antonio García Ranea [aut] (ORCID:
    <https://orcid.org/0000-0003-0327-1837>)
Maintainer: James Perkins <jimrperkins@gmail.com>
