# Generated by roxygen2: do not edit by hand

export(acc)
export(degsynth)
export(fmeasure)
export(get_org_db)
export(get_organism_table)
export(get_stats)
export(get_stats_from_cm)
export(install_DEgenes_hunter)
export(load_and_parse_gmt)
export(load_synth_dataset)
export(main_clusters_to_enrichment)
export(main_degenes_Hunter)
export(main_functional_hunter)
export(multienricher_gsea)
export(multienricher_ora)
export(multienricher_topGO)
export(parse_pseudocounts)
export(ppv)
export(recall)
export(rtable2measures)
export(spc)
export(target_generation)
export(write_clusters_to_enrichment)
export(write_df_list_as_tables)
export(write_enrich_files)
export(write_expression_report)
export(write_functional_report)
importClassesFrom(topGO,topGOdata)
importFrom(AnnotationDbi,keys)
importFrom(AnnotationDbi,select)
importFrom(Biobase,assayData)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpok)
importFrom(BiocParallel,bptry)
importFrom(DESeq2,DESeq)
importFrom(DESeq2,DESeqDataSetFromMatrix)
importFrom(DESeq2,counts)
importFrom(DESeq2,results)
importFrom(DESeq2,rlog)
importFrom(DOSE,setReadable)
importFrom(DT,datatable)
importFrom(FactoInvestigate,dimRestrict)
importFrom(FactoInvestigate,eigenRef)
importFrom(FactoMineR,HCPC)
importFrom(FactoMineR,PCA)
importFrom(FactoMineR,dimdesc)
importFrom(GO.db,GOBPANCESTOR)
importFrom(GO.db,GOBPPARENTS)
importFrom(GO.db,GOCCANCESTOR)
importFrom(GO.db,GOCCPARENTS)
importFrom(GO.db,GOMFANCESTOR)
importFrom(GO.db,GOMFPARENTS)
importFrom(GO.db,GOOBSOLETE)
importFrom(GO.db,GOTERM)
importFrom(GenomicFeatures,makeTxDbFromGFF)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(MKinfer,boot.t.test)
importFrom(Matrix,forceSymmetric)
importFrom(NOISeq,noiseqbio)
importFrom(NOISeq,readData)
importFrom(NOISeq,tmm)
importFrom(ROCR,performance)
importFrom(ROCR,prediction)
importFrom(ReactomePA,enrichPathway)
importFrom(ReactomePA,gsePathway)
importFrom(SummarizedExperiment,assay)
importFrom(WGCNA,allowWGCNAThreads)
importFrom(WGCNA,binarizeCategoricalVariable)
importFrom(WGCNA,blockwiseModules)
importFrom(WGCNA,corPvalueStudent)
importFrom(WGCNA,labels2colors)
importFrom(WGCNA,moduleEigengenes)
importFrom(WGCNA,orderMEs)
importFrom(WGCNA,pickSoftThreshold)
importFrom(WGCNA,plotDendroAndColors)
importFrom(annotatr,annotate_regions)
importFrom(annotatr,build_annotations)
importFrom(clusterProfiler,enrichGO)
importFrom(clusterProfiler,enrichKEGG)
importFrom(clusterProfiler,gseGO)
importFrom(clusterProfiler,gseKEGG)
importFrom(clusterProfiler,merge_result)
importFrom(clusterProfiler,simplify)
importFrom(data.table,as.data.table)
importFrom(data.table,merge.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setnames)
importFrom(diffcoexp,diffcoexp)
importFrom(dplyr,arrange)
importFrom(dplyr,between)
importFrom(dplyr,desc)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,row_number)
importFrom(edgeR,DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(edgeR,cpm)
importFrom(edgeR,estimateDisp)
importFrom(edgeR,exactTest)
importFrom(edgeR,glmQLFTest)
importFrom(edgeR,glmQLFit)
importFrom(edgeR,topTags)
importFrom(enrichplot,cnetplot)
importFrom(enrichplot,dotplot)
importFrom(enrichplot,emapplot)
importFrom(enrichplot,pairwise_termsim)
importFrom(fastcluster,hclust)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,fortify)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_count)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guides)
importFrom(ggplot2,scale_colour_gradientn)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_light)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylim)
importFrom(ggrepel,geom_text_repel)
importFrom(ggridges,geom_density_ridges)
importFrom(grDevices,dev.control)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,recordPlot)
importFrom(graphics,abline)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(heatmaply,heatmaply)
importFrom(knitr,knit)
importFrom(knitr,knit_expand)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,topTable)
importFrom(limma,voom)
importFrom(magrittr,"%>%")
importFrom(matrixStats,rowDiffs)
importFrom(matrixStats,rowRanges)
importFrom(matrixStats,rowVars)
importFrom(miRBaseVersions.db,miRBaseVersions.db)
importFrom(plyr,rbind.fill)
importFrom(rlang,.data)
importFrom(rmarkdown,render)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,ecdf)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,pchisq)
importFrom(stats,phyper)
importFrom(stats,pt)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rlnorm)
importFrom(stats,rnbinom)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stringr,str_remove)
importFrom(tidyr,unite)
importFrom(topGO,annFUN)
importFrom(tximport,tximport)
importFrom(utils,assignInNamespace)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,write.table)
