---
title: "ExpHunterSuite: Clusters Functional Report"
author: "SysBioLab"
output:
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container { 
        max-width: 90%;
    
    }

    .plot_real_size {
      overflow: scroll;
      max-height: 600px;
    }

    .plot_real_size img{
      max-width: none;
      max-height: none;
    }

    embed {
   
  }   
</style>

# **Functional Report**


```{r main_init, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 
if (!is.null(opt$description)){
  cat(opt$description)
}

```

```{r render_partial, echo = FALSE, warning =FALSE, message=FALSE, results = 'asis'} 

res <- list()
for(funsys in names(cl_flags_ora)) {
  if(cl_flags_ora[[funsys]] == TRUE) {
    func_clusters <- func_results[[funsys]]
    enrich_obj <- func_clusters
    exp_res <- knitr::knit_expand("ora_plots_all_objects.Rmd")
    res[[funsys]] <- knitr::knit(text=exp_res, quiet = TRUE)
  } else {
    res[[funsys]] <- paste0("## **",funsys," - Over Representation Analysis**\nNo enrichment found for ", funsys,"\n")
  }
}

cat(unlist(res), sep = '\n')

```

