```{r, include = FALSE} 
# Load necessary packages
#require(ggplot2)
#require(knitr)
#require(clusterProfiler)
```

## **Used data in this analysis**
Specifically, in this experiment set, known experiment labels are:

`r paste(knitr::knit(text = paste(sample_classes, collapse = "\n")), collapse = "\n")`

## **General description**
This report contains all the functional information that was requested by the options when functional_Hunter.R was executed.
The functional categories can be:

* KEGG pathways
* GO:
	+ Biological Process
	+ Molecular Function
	+ Cellular Component
* Reactome pathways
* Custom nomenclature

All the functional categories are computed with CluterProfiler and GO caterogires are computed also with TopGo.
Some sections will not show if there are not sinficative results. Each category is analysed using Over representation analysis (ORA) and Gene Set Analysis (GSEA).
The ORA method takes a group of significative DEGs (only DEGs, upregulated DEGs or downregulated DEGs) and performs a hypergeometric test for each term of the selected functional category.
In the case of the GSEA method, all the genes are sorted by their fold-change and the algorithm scan which genes with similar fold-change shares a term of the selected functional category.
