---
title: "ExpHunterSuite: Differential Expression Report"
output:
  html_document:
    toc: true
    toc_float: true
    df_print: paged
    fig_width: 12
---

<style type='text/css'>
    body .main-container {
        max-width: 90%;

    }

    .plot_real_size {
    	overflow: scroll;
  		max-height: 600px;
  	}

    .plot_real_size img{
    	max-width: none;
    	max-height: none;
    }

    embed {
   
  }   
</style>


```{r child=file.path(template_folder,"expression_qc.Rmd")}
```

```{r child=file.path(template_folder,"dea_results.Rmd"), eval = any(grepl("[DENL]",opt$modules))}
``` 

```{r child=file.path(template_folder,"WGCNA_results.Rmd"), eval = grepl("W", opt$modules)}
``` 
 
```{r child=file.path(template_folder,"diffcoexp_results.Rmd"), eval = grepl("X", opt$modules)}
``` 
 
```{r child=file.path(template_folder,"PCIT_results.Rmd"), eval = grepl("P", opt$modules)}
``` 

### **Values of options used to run DEGenesHunter**
First column contains the option names; second column contains the given values for each option in this run.
```{r opt_vals, echo = FALSE, warning = FALSE}
knitr::kable(cbind(opt))
```
